// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package company

import (
	"context"
	"encoding/base64"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-government-management/consts"
	"strings"
	"smart-government-management/errors"
	"smart-government-management/model"
	"smart-government-management/pb/v1"
	"github.com/jaryhe/gopkgs/util"
	"sync"
)


func certDecrypt(cert string) (string) {
	bytes,_ := base64.StdEncoding.DecodeString(cert)
	ret, _ := util.AesDecrypt(bytes, []byte("2D29BP43U3Y1B4N6REQW3F319DD23454"))
	return string(ret)
}

var areaMap = map[string]string{}
var mux sync.Mutex

func getAddrByAreaCode(code string) string {
	if len(code) < 6 {
		return ""
	}
	mux.Lock()
	defer mux.Unlock()
	if ret,ok := areaMap[code]; ok {
		return ret
	}
	p := model.AreaCode{}
	where := map[string]interface{}{
		"Code":string(code[:2])+"0000",
	}
	p.Find(database.DB(), where)
	province := p.Addr
	if string(code[2:]) == "0000" {
		if province != "" {
			areaMap[code] = province
		}
		return province
	}
	p = model.AreaCode{}
	where = map[string]interface{}{
		"Code":string(code[:4])+"00",
	}
	p.Find(database.DB(), where)
	city := p.Addr
	if string(code[4:]) == "00" {
		if province+city != "" {
			areaMap[code] = province+city
		}
		return province+city
	}

	p = model.AreaCode{}
	where = map[string]interface{}{
		"Code":code,
	}
	p.Find(database.DB(), where)
	if province+city+p.Addr != "" {
		areaMap[code] = province+city+p.Addr
	}
	return province+city+p.Addr
}

func ProjectInfo(ctx context.Context, req *v1.ProjectInfoRequest) (reply *v1.ProjectInfoReply, err error) {
	if req.Id < 1 {
		return nil, errors.ParamsError
	}
	where := map[string]interface{}{
		"ID": req.Id,
	}
	p := model.ProjectInfo{}
	p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.NoRecordError
	}
	company := model.TCompany{}
	where = map[string]interface{}{
		"ID":p.Cid,
	}
	company.Find(database.DB(), where, nil)
	reply = &v1.ProjectInfoReply{}
	reply.Info = &v1.ProjectItem{}
	reply.Info.Id = p.ID
	reply.Info.Location = p.Address
	reply.Info.Name = p.Name
	reply.Info.SafetyRecordNo = p.Safetyno
	reply.Info.Lon = p.Lng
	reply.Info.Lat = p.Lat
	reply.Info.Price = p.Invest
	reply.Info.CoveredArea = p.Buildingarea
	reply.Info.Desc = p.Description
	reply.Info.StartDay = p.Startdate.Format("2006-01-02")
	reply.Info.EndDay = p.Enddate.Format("2006-01-02")
	reply.Info.CreateTime = p.Createdat.Format("2006-01-02 15:04:05")
	reply.Info.ProjectStatus = consts.ProjectStatusM[p.Prjstatus]
	reply.Info.Category = consts.ProjectCategoryM[p.Category]
	reply.Info.ConstructType = consts.ProjectConstructTypeM[p.Constructtype]
	reply.Info.CompanyName = company.Name
	reply.Info.InvestType = p.InvestType
	reply.Info.Scale = p.Scale

	if !strings.Contains(p.Planpic, "[") {
		reply.Info.ProjectPlan = "[]"
	} else {
		reply.Info.ProjectPlan = p.Planpic
	}
	if !strings.Contains(p.Effectpic, "[") {
		reply.Info.ProjectEffectPic = "[]"
	} else {
		reply.Info.ProjectEffectPic = p.Effectpic
	}

	corp := model.ProjectCorpInfo{}
	where = map[string]interface{}{
		"ProjectId":req.Id,
	}
	corpList, err := corp.All(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply.CorpList = make([]*v1.ProjectCorpListItem, len(corpList))
	for i, v := range corpList {
		item := &v1.ProjectCorpListItem{
			CorpType:consts.ProjectCorpM[v.CorpType],
			Name:v.CorpName,
			AreaCode:getAddrByAreaCode(v.AreaCode),
			Id:v.ID,
			RegisterDate:v.RegisterDate.Format(layout),
			SocialCode:v.CorpCode,
		}
		if v.RegisterDate.Unix() <= 0 {
			item.RegisterDate = ""
		}
		reply.CorpList[i] = item
	}



	pm := model.ProjectPMInfo{}

	pmList, err := pm.All(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("ProjectList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply.PmList = make([]*v1.ProjectPMListItem, len(pmList))
	for i, v := range pmList {
		if reply.Info.Manager == "" && v.PType == consts.StaffTypeManager {
			reply.Info.Manager = v.PMName
			reply.Info.ManagerPhone = v.PMPhone
		}
		plainCert := certDecrypt(v.PMIDCardNumber)
		item := &v1.ProjectPMListItem{
			CorpType:consts.ProjectCorpM[v.CorpType],
			CorpName:v.CorpName,
			Id:v.ID,
			SocialCode:v.CorpCode,
			CertType:consts.CertTypeM[v.PMIDCardType],
			PType:consts.StaffTypeM[v.PType],
			Name:v.PMName,
			Phone:v.PMPhone,
			CertNum:plainCert,
		}
		reply.PmList[i] = item
	}


	return reply, nil
}
