// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package device

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-government-management/errors"
	"smart-government-management/model"
	"smart-government-management/pb/v1"
)

func DeviceTestedTypeInfo(ctx context.Context, req *v1.DeviceTestedTypeInfoRequest) (reply *v1.DeviceTestedTypeInfoReply, err error) {
	reply = &v1.DeviceTestedTypeInfoReply{}

	p := &model.TDeviceTypeItem{}
	m, _, err := p.AllMap(database.DB())
	if err != nil {
		logger.Error("DeviceTypeList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	ptest := &model.ProviderDeviceTesting{}
	err = ptest.Find(database.DB(), map[string]interface{}{"ID": req.Id})
	if err != nil {
		return nil, errors.DataBaseError
	}
	provider := &model.TProvider{}
	err = provider.Find(database.DB(), map[string]interface{}{"ID": ptest.ProviderId})
	if err != nil {
		return nil, errors.DataBaseError
	}
	padd := &model.TProviderDevice{}
	where := map[string]interface{}{
		"ProviderId": ptest.ProviderId,
		"DeviceCode": ptest.DeviceCode,
	}
	err = padd.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if padd.Id == 0 {
		return nil, errors.NoRecordError
	}

	testInfo := &v1.DeviceTestedTypeItem{}
	testInfo.Status = int32(ptest.Status)
	testInfo.SocialCode = provider.SocialCode
	testInfo.CreatedTime = ptest.CreatedAt.Format(layout)
	testInfo.Comment = ptest.Comment
	testInfo.ProviderName = provider.Name
	testInfo.TypeCode = int32(ptest.DeviceCode)
	testInfo.TypeName = m[int32(ptest.DeviceCode)]
	testInfo.Feedback = ptest.Feedback
	testInfo.Agent = ptest.Agent
	testInfo.AgentPhone = ptest.AgentPhone
	testInfo.Appendix = ptest.Appendix
	testInfo.Reviewer = ptest.Reviewer
	if testInfo.Appendix == "" {
		testInfo.Appendix = "[]"
	}
	testInfo.ApproveTime = ptest.ApproveTime.Format(layout)
	reply.TestApplyInfo = testInfo

	addInfo := &v1.DeviceTestedTypeItem{}
	addInfo.Status = int32(padd.Status)
	addInfo.SocialCode = provider.SocialCode
	addInfo.CreatedTime = padd.CreatedAt.Format(layout)
	addInfo.Comment = padd.Comment
	addInfo.ProviderName = provider.Name
	addInfo.TypeCode = int32(padd.DeviceCode)
	addInfo.TypeName = m[int32(padd.DeviceCode)]
	addInfo.Feedback = padd.Feedback
	addInfo.Agent = padd.Agent
	addInfo.AgentPhone = padd.AgentPhone
	addInfo.Appendix = padd.Appendix
	addInfo.Reviewer = padd.Reviewer
	if addInfo.Appendix == "" {
		addInfo.Appendix = "[]"
	}
	addInfo.ApproveTime = padd.ApproveTime.Format(layout)
	reply.AddApplyInfo = addInfo

	return reply, nil
}

/*
func DeviceTestedTypeInfo2(ctx context.Context, req *v1.DeviceTestedTypeInfoRequest) (reply *v1.DeviceTestedTypeInfoReply, err error) {
	reply = &v1.DeviceTestedTypeInfoReply{}

	p := &model.TDeviceTypeItem{}
	m, _, err := p.AllMap(database.DB())
	if err !=  nil {
		logger.Error("DeviceTypeList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}


	ptest := &model.ProviderDeviceTesting{}
	err = ptest.Find(database.DB(), map[string]interface{}{"ID":req.Id})
	if err != nil {
		return nil, errors.DataBaseError
	}
	provider := &model.TProvider{}
	err = ptest.Find(database.DB(), map[string]interface{}{"ID":ptest.ProviderId})
	if err != nil {
		return nil, errors.DataBaseError
	}
	padd := &model.TProviderDevice{}
	where := map[string]interface{}{
		"ProviderId":ptest.ProviderId,
		"DeviceCode":ptest.DeviceCode,
	}
	err = padd.FindSort(database.DB(), where, "ApproveTime desc")
	if err != nil {
		return nil, errors.DataBaseError
	}

	testInfo := &v1.DeviceTestedTypeItem{}
	testInfo.Status = int32(ptest.Status)
	testInfo.SocialCode = provider.SocialCode
	testInfo.CreatedTime = ptest.CreatedAt.Format(layout)
	testInfo.Comment = ptest.Comment
	testInfo.ProviderName = provider.Name
	testInfo.TypeCode = int32(ptest.DeviceCode)
	testInfo.TypeName = m[int32(ptest.DeviceCode)]
	testInfo.Feedback = ptest.Feedback
	testInfo.Agent = ptest.Agent
	testInfo.AgentPhone = ptest.AgentPhone
	testInfo.Appendix = ptest.Appendix
	if testInfo.Appendix == "" {
		testInfo.Appendix = "[]"
	}
	testInfo.ApproveTime = ptest.ApproveTime.Format(layout)


	addInfo := &v1.DeviceTestedTypeItem{}
	addInfo.Status = int32(padd.Status)
	addInfo.SocialCode = provider.SocialCode
	addInfo.CreatedTime = padd.CreatedAt.Format(layout)
	addInfo.Comment = padd.Comment
	addInfo.ProviderName = provider.Name
	addInfo.TypeCode = int32(padd.DeviceCode)
	addInfo.TypeName = m[int32(padd.DeviceCode)]
	addInfo.Feedback = padd.Feedback
	addInfo.Agent = padd.Agent
	addInfo.AgentPhone = padd.AgentPhone
	addInfo.Appendix = padd.Appendix
	if addInfo.Appendix == "" {
		addInfo.Appendix = "[]"
	}
	addInfo.ApproveTime = padd.ApproveTime.Format(layout)



	return reply, nil
}
*/
