// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package device

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-government-management/consts"
	"smart-government-management/errors"
	"smart-government-management/model"
	"smart-government-management/pb/v1"
	"github.com/jaryhe/gopkgs/logger"
	"go.uber.org/zap"
)
var layout = "2006-01-02 15:04:05"
func DeviceTypeList(ctx context.Context, req *v1.DeviceTypeListRequest) (reply *v1.DeviceTypeListReply, err error) {
	reply = &v1.DeviceTypeListReply{}
	p := &model.TDeviceTypeItem{}
	m, _, err := p.AllMap(database.DB())
	if err !=  nil {
		logger.Error("DeviceTypeList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if req.Page < 1 {
		req.Page = 1
	}
	mreq := model.ProviderDeviceTypeListRequest{}
	mreq.ProviderId = req.ProviderId
	// 0. 已对接的设备类型 1 设备类型申请列表 2 设备类型对接列表
	if req.ListType == 0 {
		mreq.TableName = "ProviderDeviceTesting"
		mreq.PageSize = 10
		mreq.Page = req.Page
		mreq.Filter = req.Filter
		mreq.Status = []uint32{1}
		mreq.DeviceCode = req.DeviceCode
	} else if req.ListType == 1 {
		mreq.TableName = "ProviderDevice"
		mreq.PageSize = 10
		mreq.Page = req.Page
		mreq.Filter = req.Filter
		mreq.Status = []uint32{0,2}
		mreq.DeviceCode = req.DeviceCode
	} else if req.ListType == 2 {
		mreq.TableName = "ProviderDeviceTesting"
		mreq.PageSize = 10
		mreq.Page = req.Page
		mreq.Filter = req.Filter
		mreq.Status = []uint32{0,2}
		mreq.DeviceCode = req.DeviceCode
	} else {
		return nil, errors.ParamsError
	}

	list, total, err := model.ProviderDeviceTypeList(database.DB(), &mreq)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("DeviceTypeList",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	reply.Total = total
	reply.Page = req.Page
	reply.PageSize = int32(model.PageSize)
	if total == 0 {
		return reply, nil
	}
	reply.List = make([]*v1.DeviceTypeItem, len(list))
	for i, v := range list {
		reply.List[i] = &v1.DeviceTypeItem{}
		reply.List[i].Id = v.Id
		reply.List[i].Status = int32(v.Status)
		reply.List[i].SocialCode = v.SocialCode
		reply.List[i].CreatedTime = v.ApplyTime.Format(layout)
		reply.List[i].Comment = v.Comment
		reply.List[i].ProviderName = v.ProviderName
		reply.List[i].TypeCode = int32(v.TypeCode)
		reply.List[i].TypeName = m[int32(v.TypeCode)]
		reply.List[i].Feedback = v.Feedback
		reply.List[i].Agent = v.Agent
		reply.List[i].AgentPhone = v.AgentPhone
		reply.List[i].Appendix = v.Appendix
		reply.List[i].Reviewer = v.Reviewer
		if reply.List[i].Appendix == "" {
			reply.List[i].Appendix = "[]"
		}


		if v.Status == consts.DeviceTypeStatusAddTestted || v.Status == consts.DeviceTypeStatusAddTesttedFail {
			reply.List[i].ApproveTime = v.ApproveTime.Format(layout)
		}
		if v.Status == consts.DeviceTypeStatusAddAuditted  || v.Status == consts.DeviceTypeStatusAddAudittedFail{
			reply.List[i].ApproveTime = v.ApproveTime.Format(layout)
		}
	}

	return reply, nil
}
