// Copyright 2019  githup.com. All rights reserved.
// Use of this source code is governed by  githup.com.
package provider

import (
	"context"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jaryhe/gopkgs/tasker/transtasker"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"smart-government-management/consts"
	"smart-government-management/errors"
	dbmodel "smart-government-management/model"
	"smart-government-management/pb"
	"smart-government-management/pb/v1"
	pb_v1 "smart-government-management/pb/v1"
	"time"
)

const (
	// 0 供应商入驻审批 1 设备类型申请审批 2 设备类型对接 3 拆机审批
	ApproveTypeProvider = iota
	ApproveTypeDeviceAdd
	ApproveTypeDeviceTesting
	ApproveTypeDeviceDel
)

func providerApprove(req *pb_v1.ProviderApproveRequest) (reply *pb_v1.ProviderApproveReply, err error) {
	p := &dbmodel.TProvider{}
	where := map[string]interface{}{
		"ID": req.Id,
	}
	// 状态判断
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("providerApprove",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.Id == 0 {
		return nil, errors.NoRecordError
	}
	if p.Status != consts.ProviderStatusAddAuditting {
		return nil, errors.CanNotApproveWithStatus
	}

	// 更新
	where = map[string]interface{}{
		"ID":     req.Id,
		"Status": consts.ProviderStatusAddAuditting,
	}
	value := map[string]interface{}{
		"Status":       consts.ProviderStatusAddAuditted,
		"ApproveTime": time.Now(),
	}
	if !req.Status {
		value["Status"] = consts.ProviderStatusAddAudittedFail
	}
	value["Feedback"] = req.Feedback
	err = p.Update(database.DB(), where, value)
	if err != nil {
		logger.Error("providerApprove",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	go func() {
		mreq := pb_v1.SendThroughVerifyRequest{
			Company:     p.Name,
			CompanyType: 1,
			PhoneNumber: p.BusinessContactPhone,
			Status:      req.Status,
			ApproveTime: time.Now().Format("2006-01-02 15:04:05"),
		}
		pb.Thirdparty.SendThroughVerify(context.Background(), &mreq)
	}()

	return &pb_v1.ProviderApproveReply{}, nil
}

// 设备类型对接开发者审批
func developDockingApprove(req *pb_v1.ProviderApproveRequest) (reply *pb_v1.ProviderApproveReply, err error) {

	old := &dbmodel.ProviderDeviceTesting{}


	where := map[string]interface{}{
		"ID":     req.Id,
	}
	err = old.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if old.Id == 0 {
		return nil, errors.NoRecordError
	}
	if old.Status != consts.DeviceTypeStatusAddTesting {
		return nil, errors.CanNotApproveWithStatus
	}


	p := &dbmodel.ProviderDeviceTesting{}

	// 更新
	where = map[string]interface{}{
		"ID":     req.Id,
		"Status": consts.DeviceTypeStatusAddTesting,
	}
	value := map[string]interface{}{
		"Status":               consts.DeviceTypeStatusAddTestted,
		"Feedback":              req.Feedback,
		"ApproveTime": time.Now(),
		"Reviewer":req.Reviewer,
	}
	if !req.Status {
		value["Status"] = consts.DeviceTypeStatusAddTesttedFail
	}
	err = p.Update(database.DB(), where, value)
	if err != nil {
		logger.Error("develop DockingApprove",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	return &pb_v1.ProviderApproveReply{}, nil
}

// 设配类型对接政府审批
func deviceDockingApprove(req *pb_v1.ProviderApproveRequest) (reply *pb_v1.ProviderApproveReply, err error) {
	// 开发者审批
	if req.IsDevelop && req.ApproveType == ApproveTypeDeviceTesting {
		return developDockingApprove(req)
	}

	p := &dbmodel.TProviderDevice{}
	where := map[string]interface{}{
		"ID": req.Id,
	}
	// 状态判断
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("deviceDockingApprove",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.Id == 0 {
		return nil, errors.NoRecordError
	}


	// 政府审批
	if p.Status != consts.DeviceTypeStatusAddAuditting {
		return nil, errors.CanNotApproveWithStatus
	}

	// 更新
	where = map[string]interface{}{
		"ID":     req.Id,
		"Status": consts.DeviceTypeStatusAddAuditting,
	}
	value := map[string]interface{}{
		"Status":       consts.DeviceTypeStatusAddAuditted,
		"ApproveTime": time.Now(),
		"Reviewer":req.Reviewer,
	}
	if !req.Status {
		value["Status"] = consts.DeviceTypeStatusAddAudittedFail
	}

	value["Feedback"] = req.Feedback
	err = p.Update(database.DB(), where, value)
	if err != nil {
		logger.Error("deviceDockingApprove",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}

	return &pb_v1.ProviderApproveReply{}, nil
}

/*
func deviceAddApprove(req *pb_v1.ProviderApproveRequest)(reply *pb_v1.ProviderApproveReply, err error) {
	p := &dbmodel.TDevice{}
	where := map[string]interface{}{
		"id":req.Id,
	}
	// 状态判断
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.Id == 0 {
		return nil, errors.NoRecordError
	}
	if p.VerifyStatus != consts.DeviceStatusAddProjectAuditted {
		return nil, errors.CanNotApproveWithStatus
	}

	// 更新
	where = map[string]interface{}{
		"id":req.Id,
		"verify_status":consts.DeviceStatusAddProjectAuditted,
	}
	value := map[string]interface{}{
		"verify_status":consts.DeviceStatusAddAuditted,
		"goverment_approve_time":time.Now(),
	}
	err = p.Update(database.DB(), where, value)
	if err != nil {
		return nil, errors.DataBaseError
	}
	return &pb_v1.ProviderApproveReply{}, nil
}
*/
func deviceDelApprove(req *pb_v1.ProviderApproveRequest) (reply *pb_v1.ProviderApproveReply, err error) {
	p := &dbmodel.DeviceDelJob{}
	where := map[string]interface{}{
		"ID": req.Id,
	}
	// 状态判断
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		logger.Error("deviceDelApprove",
			zap.String("err", err.Error()))
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.NoRecordError
	}
	if p.Status != consts.DeviceStatusDelAuditting {
		return nil, errors.CanNotApproveWithStatus
	}

	gdevice := &dbmodel.DeviceAll{}

	jobTask := func(db *gorm.DB) error {
		job := &dbmodel.DeviceDelJob{}
		where := map[string]interface{}{
			"ID": req.Id,
		}
		status := 2
		if req.Status {
			status = 1
		}
		value := map[string]interface{}{
			"Feedback":   req.Feedback,
			"Status":     status,
			"UpdatedAt": time.Now(),
			"Reviewer":req.Reviewer,
		}
		err := job.Update(db, where, value)
		if err != nil {
			logger.Error("deviceDelApprove",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		where = map[string]interface{}{
			"ID":p.DeviceId,
		}
		err = gdevice.Find(database.DB(), where)
		if err != nil {
			logger.Error("deviceDelApprove",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}
		return nil
	}

	delTask := func(db *gorm.DB) error {
		device := dbmodel.DeviceAll{}
		if !req.Status {
			return nil
		}
		where = map[string]interface{}{
			"ID": p.DeviceId,
		}

		err = device.Del(db, where)
		if err != nil {
			logger.Error("deviceDelApprove",
				zap.String("err", err.Error()))
			return errors.DataBaseError
		}

		if gdevice.DeviceCode == dbmodel.DeviceTypeDust {
			mreq := v1.DelDeviceRequest{Sn:gdevice.SN, DeviceCode:int32(gdevice.DeviceCode)}
			_, err := pb.Auth.DelDevice(context.Background(), &mreq)
			if err != nil {
				return err
			}
			dust := dbmodel.DustDeviceInfo{}
			where = map[string]interface{}{
				"ID":gdevice.SubId,
			}
			err = dust.Del(db, where)
			if err != nil {
				logger.Error("deviceDelApprove",
					zap.String("err", err.Error()))
				return errors.DataBaseError
			}
		}
		if gdevice.DeviceCode == dbmodel.DeviceTypeAttendance {
			attendance := dbmodel.AttendanceDeviceInfo{}
			where = map[string]interface{}{
				"ID":gdevice.SubId,
			}
			err = attendance.Del(db, where)
			if err != nil {
				logger.Error("deviceDelApprove",
					zap.String("err", err.Error()))
				return errors.DataBaseError
			}
		}
		if gdevice.DeviceCode == dbmodel.DeviceTypeVehicle {
			car := dbmodel.SlagcarDevice{}
			where = map[string]interface{}{
				"ID":device.SubId,
			}
			err = car.Del(db, where)
			if err != nil {
				logger.Error("deviceDelApprove",
					zap.String("err", err.Error()))
				return errors.DataBaseError
			}
		}
		if gdevice.DeviceCode == dbmodel.DeviceTypeVedio{
			video := dbmodel.Vssdevtbl{}
			where := map[string]interface{}{
				"DevPubID":gdevice.SN,
			}
			err = video.Del(db, where)
			if err != nil {
				logger.Error("deviceDelApprove",
					zap.String("err", err.Error()))
				return errors.DataBaseError
			}
			channel := dbmodel.Vsschanneltbl{}
			err = channel.Del(db, where)
			if err != nil {
				logger.Error("deviceDelApprove",
					zap.String("err", err.Error()))
				return errors.DataBaseError
			}
		}
		return nil
	}

	notifyTask := func(db *gorm.DB) error {
		mreq := v1.DeviceChangeNotifyRequest{
			DeviceCode: int32(gdevice.DeviceCode),
			Sn:         gdevice.SN,
		}
		_, err := pb.Auth.DeviceChangeNotify(context.Background(), &mreq)
		if err != nil {
			logger.Error("deviceDelApprove notify",
				zap.String("err", err.Error()))
			return nil
		}
		return nil
	}

	err = transtasker.Exec(database.DB(),
		transtasker.NewTask(jobTask),
		transtasker.NewTask(delTask), transtasker.NewTask(notifyTask))

	// 更新

	return &pb_v1.ProviderApproveReply{ProjectId: p.ProjectId}, err
}

func ProviderApprove(ctx context.Context, req *pb_v1.ProviderApproveRequest) (reply *pb_v1.ProviderApproveReply, err error) {
	if req.Id == 0 {
		return nil, errors.ParamsError
	}
	switch req.ApproveType {
	case ApproveTypeProvider:
		return providerApprove(req)
	case ApproveTypeDeviceAdd:
		return deviceDockingApprove(req)
	case ApproveTypeDeviceTesting:
		return deviceDockingApprove(req)
	case ApproveTypeDeviceDel:
		return deviceDelApprove(req)
	}
	return nil, errors.ParamsError
}


