package task

import (
	"context"
	"encoding/json"
	"fmt"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-government-management/consts"
	"smart-government-management/errors"
	"smart-government-management/model"
	"smart-government-management/pb/v1"
	"smart-government-management/utils"
	"strings"
	"time"
)


func TaskAdd(ctx context.Context, req *v1.TaskAddRequest)(reply *v1.TaskAddReply, err error) {

	if req.ProjectId < 1 || req.TaskName == "" || req.TaskType < 1 || req.TaskType > 2   {
		return nil, errors.ParamsError
	}
	project := model.ProjectInfo{}
	where := map[string]interface{}{
		"ID":req.ProjectId,
	}
	err = project.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if project.ID == 0 {
		return nil, errors.ProjectNotExistError
	}
	now := time.Now()
	p := model.TaskProcessingInfo{
		Code:project.Code,
		ProjectID:req.ProjectId,
		TaskName: req.TaskName,
		TaskDesc:req.TaskDesc,
		AssignmentDate:now,
		TaskType:req.TaskType,
		Creater:req.Creater,
		Status:consts.TaskStatusUnProcessing,
	}


	if req.FinishDate > 0 {
		p.FinishDate = time.Unix(req.FinishDate, 0)
	}
	if len(req.TaskImg) > 0 {
		bytes, _ := json.Marshal(req.TaskImg)
		p.TaskImg = string(bytes)
	}


	imgArray := []string{}
	if p.TaskImg != "" {
		imgArray = append(imgArray, p.TaskImg)
	}
	imgBytes,_ := json.Marshal(imgArray)
	pipe := model.TaskPipeline{
		PipeType:model.PipelineCreate,
		PipeTime:now.Format("2006-01-02 15:04:05"),
		Person:req.Creater,
		Desc:p.TaskDesc,
		Img:string(imgBytes),
		Status:"新增",
	}


	pipeArray := []model.TaskPipeline{pipe}
	pipeBytes, _ := json.Marshal(pipeArray)
	p.Pipeline = string(pipeBytes)

	for i := 0; i < 10; i++ {
		p.TaskID = fmt.Sprintf("%s%s", time.Now().Format("20060102"), utils.GenerateRandomStr(8, "char"))
		err = p.Insert(database.DB())
		if err != nil {
			if strings.Contains(strings.ToLower(err.Error()), "duplicate") {
				time.Sleep(200 *time.Microsecond)
				continue
			}
			return nil, errors.DataBaseError
		}
		break
	}
	if err != nil {
		return nil, errors.DataBaseError
	}

	return &v1.TaskAddReply{Id:p.ID}, nil
}
