package task

import (
	"context"
	"encoding/json"
	"github.com/jaryhe/gopkgs/database"
	"github.com/jinzhu/gorm"
	"smart-government-management/consts"
	"smart-government-management/errors"
	"smart-government-management/impl/v1/alarm"
	"smart-government-management/model"
	"smart-government-management/pb/v1"
	"time"
)


func TaskApprove(ctx context.Context, req *v1.TaskApproveRequest)(reply *v1.TaskApproveReply, err error) {

	if req.Id < 1 || req.Reviewer == "" {
		return nil, errors.ParamsError
	}
	task := model.TaskProcessingInfo{}
	where := map[string]interface{}{
		"ID":req.Id,
	}
	err = task.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if task.ID == 0 {
		return nil, errors.NoRecordError
	}

	if task.Status != consts.TaskStatusUnApprove {
		return nil, errors.CanNotApproveWithStatus
	}
	now := time.Now()

	values := map[string]interface{}{
		"Status":consts.TaskStatusProcessed,
		"Feedback":req.Feedback,
		"Reviewer":req.Reviewer,
		"ApproveDate":now,
		"ApproveImg":"[]",
	}
	img := "[]"
	if !req.Status {
		values["Status"] = consts.TaskStatusApprovedRefuse
	}
	if len(req.ApproveImg) > 0 {
		bytes, _ := json.Marshal(req.ApproveImg)
		values["ApproveImg"] = string(bytes)
		img = string(bytes)
	}

	pipeArray := []model.TaskPipeline{}
	json.Unmarshal([]byte(task.Pipeline), &pipeArray)

	pipe := model.TaskPipeline{
		PipeType:model.PipelineApprove,
		PipeTime:now.Format("2006-01-02 15:04:05"),
		Person:req.Reviewer,
		Desc:req.Feedback,
		Img:img,
		//ApproveStatus:req.Status,
	}
	if req.Status {
		pipe.Status = "通过"
	} else {
		pipe.Status = "不通过"
	}
	pipeArray = append(pipeArray, pipe)
	pipeBytes, _ := json.Marshal(pipeArray)
	values["Pipeline"] = string(pipeBytes)

	db := database.DB().Begin()
	if task.TaskType == 3 && req.Status {
		mreq := v1.AlarmHandleRequest{
			Id:task.ID,
			Handle:true,
		}
		err = alarm.AlarmHandleByTask(&mreq,db)
		if err != nil {
			db.Rollback()
			return nil, err
		}
	}

	p := model.TaskProcessingInfo{}

	err = p.Update(db, where, values)

	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}
	db.Commit()
	return &v1.TaskApproveReply{}, nil
}
