// Copyright 2019  githup.com. All rights reserved.
// Use of this source code is governed by  githup.com.
package model


import (
	"fmt"
	"github.com/jinzhu/gorm"
	"time"
)

type TProviderDevice struct {
	Id int64 `gorm:"column:ID" json:"id" form:"id"`
	ProviderId int64 `gorm:"column:ProviderId" json:"provider_id" form:"provider_id"`
	DeviceCode int32 `gorm:"column:DeviceCode" json:"device_code" form:"device_code"`
	Status int32 `gorm:"column:Status" json:"status" form:"status"`
	CreatedAt time.Time `gorm:"column:CreatedAt"`
	Feedback string `gorm:"column:Feedback"`
	Comment string `gorm:"column:Comment"`
	Appendix string `gorm:"column:Appendix"`
	Agent string `gorm:"column:Agent"`
	AgentPhone string `gorm:"column:AgentPhone"`
	ApproveTime time.Time `gorm:"column:ApproveTime"`
	Reviewer string `gorm:"column:Reviewer" json:"reviewer"`
}

func (TProviderDevice) TableName() string {
	return "ProviderDevice"
}

func (p *TProviderDevice) Insert(db *gorm.DB) error {
	return db.Create(p).Error
}

func (p *TProviderDevice) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).First(p).Error
}

func (p *TProviderDevice) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	return db.Table(p.TableName()).Table(p.TableName()).Where(cond, val...).Updates(values).Error
}

func (p *TProviderDevice) FindSort(db *gorm.DB, where map[string]interface{}, sort string) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	ps := []TProviderDevice{}
	err = db.Table(p.TableName()).Where(cond, val...).Order(sort).Limit(1).Find(&ps).Error
	if err != nil {
		return err
	}
	if len(ps) > 0 {
		*p = ps[0]
	}
	return nil
}


func (p *TProviderDevice) Save(db *gorm.DB) error {
	return db.Save(p).Error
}


func (p *TProviderDevice) List(db *gorm.DB, where map[string]interface{}) (list []TProviderDevice, err error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		result := db.Table(p.TableName()).Where(cond, val...).Find(&list)

		return list, result.Error
	}
	result := db.Table(p.TableName()).Find(&list)

	return list, result.Error
}

func (p *TProviderDevice) ListSort(db *gorm.DB, where map[string]interface{}, sort string) (list []TProviderDevice, err error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		result := db.Table(p.TableName()).Where(cond, val...).Order(sort).Find(&list)

		return list, result.Error
	}
	result := db.Table(p.TableName()).Order(sort).Find(&list)

	return list, result.Error
}

func (p *TProviderDevice) All(db *gorm.DB) (list []TProviderDevice, err error) {

	result := db.Table(p.TableName()).Find(&list)

	return list, result.Error
}

func (p *TProviderDevice) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error

		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	return ret, err
}

func (p *TProviderDevice) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}


type ProviderDeviceTypeInfo struct {
	Id int64
	Status uint32
	SocialCode string
	ApplyTime time.Time
	ApproveTime time.Time
	Comment string
	ProviderName string
	TypeCode uint32
	Feedback string
	Appendix string
	Agent string
	AgentPhone string
	Reviewer string
}

func getWhereString(array []string)string {
	ret := ""
	for _, v := range array {
		if ret == "" {
			ret = fmt.Sprintf("where %s", v)
			continue
		}
		ret = fmt.Sprintf("%s and %s", ret, v)
	}
	return ret
}

type ProviderDeviceTypeListRequest struct {
	Page int32
	PageSize int32
	Filter string
	Status []uint32
	TableName string
	ProviderId int64
	DeviceCode int32
}

func ProviderDeviceTypeListSql(req * ProviderDeviceTypeListRequest) (string, string, []interface{}) {
	sql := fmt.Sprintf("select t1.ID as id, t1.Reviewer as reviewer, t1.Status as status, t1.Comment as comment, t1.CreatedAt as apply_time, t1.ApproveTime as approve_time, t1.DeviceCode as type_code, t1.Agent as agent, t1.AgentPhone as agent_phone, t1.Appendix as appendix, t2.Name as provider_name, t2.SocialCode as social_code "+
		" from %s as t1 left join Provider as t2 on t1.ProviderId=t2.ID", req.TableName)
	countSql := fmt.Sprintf("select count(1) as count from %s as t1 left join Provider as t2 on t1.ProviderId=t2.ID", req.TableName)
	whereArray := []string{}
	args := []interface{}{}
	if len(req.Status) >= 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.Status in(?)"))
		args = append(args, req.Status)
	}
	if req.Filter != "" {
		whereArray = append(whereArray, fmt.Sprintf("(t2.Name like '%%%s%%' or t2.SocialCode like '%%%s%%')", req.Filter, req.Filter))
	}
	if req.ProviderId > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t2.ID=%d", req.ProviderId))

	}
	if req.DeviceCode > 0 {
		whereArray = append(whereArray, fmt.Sprintf("t1.DeviceCode=%d", req.DeviceCode))
	}
	where := getWhereString(whereArray)

	offset := int64(req.Page - 1)*int64(req.PageSize)
	sql = fmt.Sprintf("%s %s order by t1.CreatedAt limit %d offset %d", sql, where, req.PageSize, offset)
	countSql = fmt.Sprintf("%s %s", countSql, where)
	return sql, countSql, args
}


func ProviderDeviceTypeList(db *gorm.DB, req * ProviderDeviceTypeListRequest) ([]ProviderDeviceTypeInfo, int64, error) {
	type ResultCount struct {
		Count int64
	}

	array := []ResultCount{}
	sql, countSql, args := ProviderDeviceTypeListSql(req)
	err := db.Raw(countSql, args...).Scan(&array).Error
	if err != nil {
		return nil, 0, err
	}
	if len(array) == 0 {
		return nil, 0, nil
	}
	if array[0].Count == 0 {
		return nil, 0, nil
	}
	list := []ProviderDeviceTypeInfo{}
	err = db.Raw(sql, args...).Scan(&list).Error
	if err != nil {
		return nil, 0, err
	}

	return list, array[0].Count, nil

}
