// Copyright 2019  github.com. All rights reserved.
// Use of this source code is governed by  github.com.
package model

import (
	"fmt"
	"github.com/jinzhu/gorm"
	"time"
)

const (
	StaffAttendanceIn = 1
	StaffAttendanceOut = 2
)

type TStaffAttendance struct {
	ID         int64  `gorm:"column:ID;primary_key"`
	Code string `gorm:"column:Code" json:"code"`
	TeamSysNo int `gorm:"column:TeamSysNo" json:"team_sys_no"`
	TeamName string `gorm:"column:TeamName" json:"team_name"`
	WorkName string `gorm:"column:WorkName" json:"work_name"`
	IDCardType int `gorm:"column:IDCardType" json:"id_card_type"`
	IDCardNumber string `gorm:"column:IDCardNumber" json:"id_card_number"`
	Date time.Time `gorm:"column:Date" json:"date"`
	Direction int `gorm:"column:Direction" json:"direction"`
	Temperature float64 `gorm:"column:Temperature" json:"temperature"`
	ImageURL string `gorm:"column:ImageURL" json:"image_url"`
	AttendType int `gorm:"column:AttendType" json:"attend_type"`
	Lng float64 `gorm:"column:Lng" json:"lng"`
	Lat float64 `gorm:"column:Lat" json:"lat"`
	ProjectId  int64  `gorm:"column:ProjectId" json:"project_id"`
	WorkNo     string `gorm:"column:WorkNo" json:"work_no"`
	Sn string `gorm:"column:Sn" json:"sn"`
	TabType int `gorm:"-"` // 用于判断表0 t_staff_attendance, 1-12 t_staff_attendance_monthx, 13 t_staff_attendance_day
}

const (
	AttendanceTabDay = iota
	AttendanceTabMonth1
	AttendanceTabMonth2
	AttendanceTabMonth3
	AttendanceTabMonth4
	AttendanceTabMonth5
	AttendanceTabMonth6
	AttendanceTabMonth7
	AttendanceTabMonth8
	AttendanceTabMonth9
	AttendanceTabMonth10
	AttendanceTabMonth11
	AttendanceTabMonth12
)

func (p TStaffAttendance) TableName() string {

	if p.TabType >=AttendanceTabMonth1 && p.TabType <= AttendanceTabMonth12 {
		return fmt.Sprintf("db_smart_attendance.WorkerAttendanceInfoMonth%d", p.TabType)
	}
	return "db_smart_attendance.WorkerAttendanceInfoDay"
}

func (p *TStaffAttendance) Insert(db *gorm.DB) error {
	return db.Create(p).Error
}


func (p *TStaffAttendance) Del(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).Delete(p).Error
}

func (p *TStaffAttendance) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	return db.Table(p.TableName()).Where(cond, val...).First(p).Error
}

func (p *TStaffAttendance) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	return db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
}


func (p *TStaffAttendance) Save(db *gorm.DB) error {
	return db.Save(p).Error
}

func (p *TStaffAttendance) Count(db *gorm.DB, where map[string]interface{}) (int64, error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return 0, err
		}
		ret := int64(0)
		err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error

		return ret, err
	}
	ret := int64(0)
	err := db.Table(p.TableName()).Count(&ret).Error
	return ret, err
}

func (p *TStaffAttendance) List(db *gorm.DB, where map[string]interface{}, page int) (list []TStaffAttendance, err error) {
	if len(where) > 0 {
		cond, val, err := whereBuild(where)
		if err != nil {
			return list, err
		}

		result := db.Table(p.TableName()).Where(cond, val...).Limit(PageSize).Offset(page).Find(&list)

		return list, result.Error
	}
	result := db.Table(p.TableName()).Limit(10).Offset(page).Find(&list)

	return list, result.Error

}

func (p *TStaffAttendance) AttendanceCount(db *gorm.DB, t string) (int64, error) {
	type Result struct {
		Count int64
	}
	array := []Result{}
	sql := fmt.Sprintf("select 1 from db_smart_attendance.WorkerAttendanceInfoDay where Direction=1 and DATE(Date)='%s' group by WorkNo", t)
	sql = fmt.Sprintf("select count(1) as count from (%s) a", sql)
	err := db.Raw(sql).Scan(&array).Error
	if err != nil {
		return 0, err
	}
	if len(array) > 0 {
		return array[0].Count, nil
	}
	return 0, nil
}
